#
#	 This file is part of stov, written by Helmut Pozimski in 2012.
#
# 	stov is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, version 2 of the License.
#
#	stov is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with stov.  If not, see <http://www.gnu.org/licenses/>.


# -*- coding: utf8 -*-

import sys
import gettext
import urllib2
import sqlite3

import feedparser

import youtube


class sub(object):
	def __init__(self, type, name, conf, search="", id=0, title="", directory=""):
		self.__ID = id
		self.__title = title
		self.__type = type
		self.__name = name
		self.__search = search
		self.__directory = directory
		self.__APIURL = ""
		self.__conf = conf
		self.DownloadedVideos = []

		if self.__name != "delete":
			self.__ConstructAPIURL()

	def GetTitle(self):
		return self.__title

	def Delete(self):

		"""Deletes all videos from the subscription from the database and deletes
		the subscription itself once it's done.

		"""
		try:
			self.__connection = sqlite3.connect(self.__conf.dbpath)
		except sqlite3.OperationalError:
			print >> sys.stderr, _("Could not write to database, subscription "
						"was NOT deleted. Please check permissions and try again.")
		else:
			self.__cursor = self.__connection.cursor()
			self.__deletevideos = "DELETE FROM videos WHERE subscription_id=?"
			self.__cursor.execute(self.__deletevideos, (self.__ID,))
			self.__deletesubscription = "DELETE FROM subscriptions WHERE id=?"
			self.__cursor.execute(self.__deletesubscription, (self.__ID,))
			self.__connection.commit()

		print _("Subscription deleted successfully!")
		self.__connection.close()

	def UpdateVideos(self):
		"""Retrieves the current videos from the youtube API and adds them
		to the database if they where not already added earlier

		"""
		self.__ParseAPIData()
		self.__processed = []
		try:
			self.__connection = sqlite3.connect(self.__conf.dbpath)
			self.__cursor = self.__connection.cursor()
		except sqlite3.OperationalError:
			print >> sys.stderr, _("Could not write to database, "
						"new video was NOT added! Please check permissions and try again.")

		for i in self.__ParsedResponse.entries:
			self.__ytid = i.link.partition("=")
			self.__ytid = self.__ytid[2].partition("&")
			self.__ytid = self.__ytid[0]
			self.__videoquery = "SELECT id FROM videos WHERE ytid=?"
			self.__cursor.execute(self.__videoquery, (self.__ytid,))
			self.__tmpid = self.__cursor.fetchall()
			if self.__tmpid == []:
				self.__query = "INSERT INTO videos (title, description, \
				ytid, subscription_id, downloaded) VALUES (?, ?, ?, ?, ?)"
				self.__data = (i.title, i.description, self.__ytid, self.__ID, 0)
				self.__cursor.execute(self.__query, self.__data)

		self.__connection.commit()
		self.__connection.close()

	def GetVideos(self):
		"""Retrieves all videos in the subscription from the database and saves
		them in an the internal list so they can be accessed by the object

		"""
		self.__videos = []
		try:
			self.__connection = sqlite3.connect(self.__conf.dbpath)
			self.__cursor = self.__connection.cursor()
		except sqlite3.OperationalError:
			print >> sys.stderr, _("Could not access database. "
						"Please check permissions and try again.")

		self.__videoquerybysubscription = "SELECT id, title, description, \
		ytid, downloaded FROM videos WHERE subscription_id=?"
		self.__cursor.execute(self.__videoquerybysubscription, (self.__ID,))
		self.__videodata = self.__cursor.fetchall()
		for i in self.__videodata:
			self.__videos.append(youtube.video(id=i[0],
			title=i[1], description=i[2], ytid=i[3],
			downloaded=i[4], conf=self.__conf))

		self.__connection.close()

	def DownloadVideos(self):
		"""Uses the DownloadVideo method of the video object to download all
		videos contained in the subscription and adds them to the list of
		downloaded videos if the download succeeds.

		"""
		for video in self.__videos:
			if video.DownloadVideo(self.__directory) is True:
				self.DownloadedVideos.append(video.title)

	"""Prints a list of all videos contained in the subscription"""
	def PrintVideos(self):
		print _("Videos in subscription ") + self.__title + ":\n"
		for i in self.__videos:
			print i.title

	def AddSub(self):
		"""Adds a new subscription to the database"""
		self.__ParseAPIData()
		try:
			self.__connection = sqlite3.connect(self.__conf.dbpath)
		except sqlite3.OperationalError:
			print >> sys.stderr, _("Could not write to database, new "
						"subscription was NOT added! Please check permissions and try again.")
		else:
			self.__cursor = self.__connection.cursor()
			self.__title = self.__ParsedResponse.feed.title
			self.__directory = self.__name + "_" + self.__search.replace(" ", "_")
			self.__query = "INSERT INTO subscriptions (title, type, searchstring, \
			directory, name) VALUES (?, ?, ?, ?, ?)"
			self.__data = (self.__title, self.__type, self.__search,
			self.__directory, self.__name)
			self.__cursor.execute(self.__query, self.__data)
			self.__connection.commit()

		self.__query = "SELECT id from subscriptions where title=?"
		self.__data = (self.__title, )
		self.__cursor.execute(self.__query, self.__data)
		self.__ID = self.__cursor.fetchone()
		self.__ID = self.__ID[0]
		self.__connection.close()

	def __ParseAPIData(self):
		"""Retrieves the youtube API data and parses them so they can be used
		by the object

		"""
		try:
			self.__APIResponse = urllib2.urlopen(self.__APIURL)
		except IOError:
			print >> sys.stderr, _("Could not get API data, maybe the API "
						"is down or you have given wrong parameters"
						", please check and try again!")
			sys.exit(1)
		else:
			self.__ParsedResponse = feedparser.parse(self.__APIResponse)

	def __ConstructAPIURL(self):
		"""Constructs the API URL which is used to retrieve API data"""
		if self.__type == "channel":
			self.__APIURL = "https://gdata.youtube.com/feeds/api/users/"\
			+ urllib2.quote(self.__name) + "/uploads/"
			if self.__search != "":
				self.__APIURL = self.__APIURL + "?q=" + "%22"\
				+ urllib2.quote(self.__search) + "%22"
		elif self.__type == "search":
			self.__APIURL = "http://gdata.youtube.com/feeds/api/videos?q="\
			+ urllib2.quote(self.__search) + "&max-results=10"
		elif self.__type == "playlist":
			self.__APIURL = "https://gdata.youtube.com/feeds/api/playlists/"\
			+ "%20" + urllib2.quote(self.__name) + "%20"
		else:
			print >> sys.stderr, _("None or invalid subscription type given, "
						"please check the type option and try again")
